@@ -190,10 +190,7 @@ def compare_derivatives(func, numerical_derivatives,
                                               num_deriv_value, 1e-4):
 
                             # It is possible that the result is NaN:
-
-                            # ! Python 2.6+: this would be
-                            # not math.isnan(func_approx):
-                            if func_approx == func_approx:
+                            if isnan(func_approx):
                                 raise DerivativesDiffer(
                                     "Derivative #%d of function '%s' may be"
                                     " wrong: at args = %s,"
@@ -278,6 +275,19 @@ def test_value_construction():
     assert x.std_dev == x2.std_dev
     assert x.tag == 'pi'
     assert x2.tag == 'pi'
+
+    # Negative standard deviations should be caught in a nice way
+    # (with the right exception):
+    try:
+        x = ufloat(3, -0.1)
+    except uncertainties.NegativeStdDev:
+        pass
+    
+    try:
+        # Obsolete form:
+        x = ufloat((3, -0.1))
+    except uncertainties.NegativeStdDev:
+        pass
     
 def test_ufloat_fromstr():
     "Input of numbers with uncertainties as a string"
@@ -324,7 +334,10 @@ def test_ufloat_fromstr():
         
         # NaN uncertainty:
         u'(3.141±nan)E+02': (314.1, float('nan')),
+        '3.141e+02+/-nan': (314.1, float('nan')),
         '3.4(nan)e10': (3.4e10, float('nan')),
+        # NaN value:
+        'nan+/-3.14e2': (float('nan'), 314),
         # "Double-floats"
         '(-3.1415 +/- 1e-4)e+200': (-3.1415e200, 1e196),
         '(-3.1415e-10 +/- 1e-4)e+200': (-3.1415e190, 1e196),
@@ -1448,10 +1461,7 @@ def power_special_cases(op):
     assert op(zero, p) == 0
 
     # The outcome of 1**nan and nan**0 was undefined before Python
-    # 2.6 (http://docs.python.org/library/math.html#math.pow):
-    if sys.version_info >= (2, 6):
-        assert op(float('nan'), zero) == 1.0
-        assert op(one, float('nan')) == 1.0
+    # 2.6 (http://docs.python.org/library/math.html#math.pow).
         
     # …**0 == 1.0:
     assert op(p, 0) == 1.0        
@@ -1544,21 +1554,12 @@ def test_repr():
     x = ufloat(3, 1, "length")
     assert repr(x) == '< length = 3.0+/-1.0 >'
 
-def python26_add(dict0, dict1):
-    '''
-    If Python 2.6+ is running, Updates dict0 with dict1 and returns the
-    updated dict0.
-    '''
-    if sys.version_info >= (2, 6):
-        dict0.update(dict1)
-    return dict0
-    
 def test_format():
     '''Test the formatting of numbers with uncertainty.'''
 
-    # The way NaN is formatted with F and E depends on the version of
-    # Python (NAN for Python 2.7+):
-    NaN_EF = '%F' % float('nan')
+    # The way NaN is formatted with F, E and G depends on the version
+    # of Python (NAN for Python 2.7+):
+    NaN_EFG = '%F' % float('nan')
     
     # Tests of each point of the docstring of
     # AffineScalarFunc.__format__() in turn, mostly in the same order.
@@ -1579,15 +1580,10 @@ def test_format():
         },
         
         # Full generalization of float formatting:
-        (3.1415, 0.0001): python26_add({
+        (3.1415, 0.0001): {
             '+09.2uf': '+03.14150+/-000.00010'
-        }, {
-            # Alignment is not available with the % formatting
-            # operator of Python < 2.6:
-            '*^+9.2uf': '+3.14150*+/-*0.00010*',
-            '>9f': '  3.14150+/-  0.00010'  # Width and align
-        }),
-
+        },
+        
         # Number of digits of the uncertainty fixed:
         (123.456789, 0.00123): {
             '.1uf': '123.457+/-0.001',
@@ -1734,38 +1730,33 @@ def test_format():
         # Python 3.2 and 3.3 give 1.4e-12*1e+12 = 1.4000000000000001
         # instead of 1.4 for Python 3.1. The problem does not appear
         # with 1.2, so 1.2 is used.
-        (-1.2e-12, 0): python26_add({
-            '12.2gPL': ur'  -1.2×10⁻¹²±           0'
-        }, {
-            # Pure "width" formats are not accepted by the % operator,
-            # and only %-compatible formats are accepted, for Python <
-            # 2.6:
+        (-1.2e-12, 0): {
+            '12.2gPL': ur'  -1.2×10⁻¹²±           0',
             '13S': '  -1.2(0)e-12',
             '10P': u'-1.2×10⁻¹²±         0',
-            'L': r'\left(-1.2 \pm 0\right) \times 10^{-12}',
+            'L': r'\left(-1.2 \pm 0\right) \times 10^{-12}',            
             # No factored exponent, LaTeX
             '1L': r'-1.2 \times 10^{-12} \pm 0',
             'SL': r'-1.2(0) \times 10^{-12}',
             'SP': ur'-1.2(0)×10⁻¹²'
-        }),
+        },
 
         # Python 3.2 and 3.3 give 1.4e-12*1e+12 = 1.4000000000000001
         # instead of 1.4 for Python 3.1. The problem does not appear
         # with 1.2, so 1.2 is used.        
-        (-1.2e-12, float('nan')): python26_add({
-            '.2uG': '(-1.2+/-%s)E-12' % NaN_EF,  # u ignored, format used
-            '15GS': '  -1.2(%s)E-12' % NaN_EF
-        }, {
+        (-1.2e-12, float('nan')): {
+            '.2uG': '(-1.2+/-%s)E-12' % NaN_EFG,  # u ignored, format used
+            '15GS': '  -1.2(%s)E-12' % NaN_EFG,
             'SL': r'-1.2(\mathrm{nan}) \times 10^{-12}',  # LaTeX NaN
             # Pretty-print priority, but not for NaN:
             'PSL': u'-1.2(\mathrm{nan})×10⁻¹²',
             'L': r'\left(-1.2 \pm \mathrm{nan}\right) \times 10^{-12}',
             # Uppercase NaN and LaTeX:
             '.1EL': (r'\left(-1.2 \pm \mathrm{%s}\right) \times 10^{-12}'
-                     % NaN_EF),
+                     % NaN_EFG),
             '10': '  -1.2e-12+/-       nan',
             '15S': '  -1.2(nan)e-12'
-        }),
+        },
 
         (3.14e-10, 0.01e-10): {
             # Character (Unicode) strings:
@@ -1777,17 +1768,16 @@ def test_format():
         },
     
         # Some special cases:
-        (1, float('nan')): python26_add({
+        (1, float('nan')): {
             'g': '1+/-nan',
-            'G': '1+/-%s' % NaN_EF,
+            'G': '1+/-%s' % NaN_EFG,
             '%': '(100.000000+/-nan)%',  # The % format type is like f
             # Should be the same as '+05', for floats, but is not, in
             # Python 2.7:
             '+05g': '+0001+/-00nan',
             # 5 is the *minimal* width, 6 is the default number of
             # digits after the decimal point:
-            '+05%': '(+100.000000+/-00nan)%'
-        }, {
+            '+05%': '(+100.000000+/-00nan)%',
             # There is a difference between '{}'.format(1.) and
             # '{:g}'.format(1.), which is not fully obvious in the
             # documentation, which indicates that a None format type
@@ -1795,44 +1785,30 @@ def test_format():
             # actually interpreted as str(), and that str() does not
             # have to behave like g ('{}'.format(1.234567890123456789)
             # and '{:g}'.format(1.234567890123456789) are different).
-            '': '1.0+/-nan',
-            # This is ugly, but consistent with
-            # '{:+05}'.format(float('nan')) and format(1.) [which
-            # differs from format(1)!):
-            '+05': '+01.0+/-00nan'            
-            }),
+            '': '1.0+/-nan'            
+        },
         
         (9.9, 0.1): {
             '.1ue': '(9.9+/-0.1)e+00',
             '.0fS': '10(0.)'
         },
+        
         (9.99, 0.1): {
              # The precision has an effect on the exponent, like for
              # floats:
             '.2ue': '(9.99+/-0.10)e+00',  # Same exponent as for 9.99 alone
             '.1ue': '(1.00+/-0.01)e+01'  # Same exponent as for 9.99 alone
         },
+        
         # 0 uncertainty: nominal value displayed like a float:
-        (1.2345, 0): python26_add({
+        (1.2345, 0): {
             '.2ue': '(1.23+/-0)e+00',
-            '1.2ue': '1.23e+00+/-0',
+            '1.2ue': '1.23e+00+/-0',  # No factored exponent
             '.2uf': '1.23+/-0',
             '.2ufS': '1.23(0)',
             '.2fS': '1.23(0)',
             'g': '1.2345+/-0'
-        }, {
-            '': '1.2345+/-0'
-        }),
-
-        # Alignment and filling characters:
-        (3.1415e10, 0): python26_add(
-        {}, {
-            '<15': '3.1415e+10     +/-0              ',
-            '<20S': '3.1415(0)e+10       ',
-            # Trying to trip the format parsing with a fill character
-            # which is an alignment character:
-            '=>15': '=====3.1415e+10+/-==============0'
-        }),
+        },
         
         (1234.56789, 0): {
             '1.2ue': '1.23e+03+/-0',  # u ignored
@@ -1882,7 +1858,41 @@ def test_format():
             '13.6g': '  1.20000e-34+/-  0.00000e-34',
             '13.6G': '  1.20000E-34+/-  0.00000E-34',
             '.6GL': r'\left(1.20000 \pm 0.00000\right) \times 10^{-34}'
+        },
+        (float('nan'), 100): {  # NaN *nominal value*
+            '': 'nan+/-100.0',  # Like '{}'.format(100.)
+            'g': 'nan+/-100',  # Like '{:g}'.format(100.)
+            '.1e': '(nan+/-1.0)e+02',  # Similar to 1±nan
+            '.1E': '(%s+/-1.0)E+02' % NaN_EFG,
+            '.1ue': '(nan+/-1)e+02',
+            '10.1e': '       nan+/-   1.0e+02'
+        },
+        (float('nan'), 1e8): {  # NaN *nominal value*
+            '': '(nan+/-1.0)e+08',  # Like '{}'.format(1.)
+            'g': '(nan+/-1)e+08',  # Like '{:g}'.format(1.)
+            '.1e': '(nan+/-1.0)e+08',
+            '.1E': '(%s+/-1.0)E+08' % NaN_EFG,
+            '.1ue': '(nan+/-1)e+08',
+            '10.1e': '       nan+/-   1.0e+08'  # 'nane+08' would be strange
+        },                
+        (float('nan'), 123456789): {  # NaN *nominal value*
+            '': '(nan+/-1.23456789)e+08',  # Similar to '{}'.format(123456789.)
+            'g': '(nan+/-1.23457)e+08',  # Similar to '{:g}'.format(123456789.)
+            '.1e': '(nan+/-1.2)e+08',
+            '.1E': '(%s+/-1.2)E+08' % NaN_EFG,
+            '.1ue': '(nan+/-1)e+08',
+            '.1ueL': r'\left(\mathrm{nan} \pm 1\right) \times 10^{8}',
+            '10.1e': '       nan+/-   1.2e+08',
+            '10.1eL': r'\mathrm{nan} \pm 1.2 \times 10^{8}'
+        },        
+        (float('nan'), float('nan')): {  # *Double* NaN
+            '': 'nan+/-nan',
+            '.1e': 'nan+/-nan',
+            '.1E': '%s+/-%s' % (NaN_EFG, NaN_EFG),
+            '.1ue': 'nan+/-nan',
+            'EL': r'\mathrm{%s} \pm \mathrm{%s}' % (NaN_EFG, NaN_EFG)
         }
+        
     }
 
     # ',' format option: introduced in Python 2.7
@@ -1911,7 +1921,7 @@ def test_format():
 
         for (format_spec, result) in representations.iteritems():
 
-            # print "FORMATTING", repr(value), "WITH", format_spec
+            ## print "FORMATTING %s WITH '%s'" % (repr(value), format_spec)
             
             # Jython 2.5.2 does not always represent NaN as nan or NAN
             # in the CPython way: for example, '%.2g' % float('nan')
